@@ -25,6 +25,7 @@ from jstreams.stream import (
     MapperWith,
     Reducer,
     reducerOf,
+    isEmptyOrNone,
 )
 
 from jstreams.tryOpt import (
@@ -40,9 +41,10 @@ from jstreams.rx import (
     BehaviorSubject,
     PublishSubject,
     ReplaySubject,
-)
-
-from jstreams.rxops import (
+    CompletedHandler,
+    ErrorHandler,
+    DisposeHandler,
+    NextHandler,
     Pipe,
     Reduce,
     Filter,
@@ -53,12 +55,25 @@ from jstreams.rxops import (
     RxOperator,
     BaseFilteringOperator,
     BaseMappingOperator,
+    rxTake,
+    rxTakeWhile,
+    rxTakeUntil,
+    rxDropWhile,
+    rxDropUntil,
+    rxDrop,
+    TakeWhile,
+    TakeUntil,
+    Drop,
+    DropUntil,
+    DropWhile,
+    Take,
 )
 
 from jstreams.thread import (
     LoopingThread,
     CallbackLoopingThread,
     cancelThread,
+    Cancellable,
 )
 
 from jstreams.timer import (
@@ -76,6 +91,15 @@ from jstreams.ioc import (
     AutoStart,
     inject,
     var,
+    InjectedDependency,
+    OptionalInjectedDependency,
+    resolveDependencies,
+    resolveVariables,
+    Variable,
+    StrVariable,
+    IntVariable,
+    FloatVariable,
+    Dependency,
 )
 
 from jstreams.noop import (
@@ -138,6 +162,15 @@ from jstreams.predicate import (
     anyOf,
     allOf,
     noneOf,
+    Not,
+    NotStrict,
+    hasKey,
+    hasValue,
+    isInInterval,
+    isInOpenInterval,
+    isKeyIn,
+    isValueIn,
+    contains,
 )
 
 from jstreams.match import (
@@ -150,6 +183,16 @@ from jstreams.match import (
     defaultCase,
 )
 
+from jstreams.tuples import (
+    Pair,
+    pair,
+    Triplet,
+    triplet,
+    leftMatches,
+    rightMatches,
+    middleMatches,
+)
+
 __all__ = [
     "each",
     "dictUpdate",
@@ -270,4 +313,47 @@ __all__ = [
     "MapperWith",
     "Reducer",
     "reducerOf",
+    "Not",
+    "NotStrict",
+    "hasKey",
+    "hasValue",
+    "isInInterval",
+    "isInOpenInterval",
+    "contains",
+    "rxTake",
+    "rxTakeWhile",
+    "rxTakeUntil",
+    "rxDropWhile",
+    "rxDropUntil",
+    "rxDrop",
+    "Cancellable",
+    "isEmptyOrNone",
+    "CompletedHandler",
+    "ErrorHandler",
+    "DisposeHandler",
+    "NextHandler",
+    "TakeWhile",
+    "TakeUntil",
+    "Drop",
+    "DropUntil",
+    "DropWhile",
+    "Take",
+    "Pair",
+    "pair",
+    "Triplet",
+    "triplet",
+    "leftMatches",
+    "rightMatches",
+    "middleMatches",
+    "isKeyIn",
+    "isValueIn",
+    "InjectedDependency",
+    "OptionalInjectedDependency",
+    "resolveDependencies",
+    "resolveVariables",
+    "Variable",
+    "StrVariable",
+    "IntVariable",
+    "FloatVariable",
+    "Dependency",
 ]
